@@ -31,12 +31,13 @@
 #include "kerfuffle_export.h"
 
 #include <QString>
-#include <QStringList>
 #include <QHash>
 #include <QObject>
 #include <QVariant>
 #include <QDebug>
 
+#include <KServiceTypeTrader>
+
 class KJob;
 
 namespace Kerfuffle
@@ -78,6 +79,12 @@ enum EntryMetaDataType {
     Custom = 1048576
 };
 
+enum ArchiveError {
+    NoError = 0,
+    NoPlugin,
+    FailedPlugin
+};
+
 typedef QHash<int, QVariant> ArchiveEntry;
 
 /**
@@ -101,11 +108,11 @@ struct fileRootNodePair
     fileRootNodePair()
     {}
 
-    fileRootNodePair(QString f)
+    fileRootNodePair(const QString &f)
         : file(f)
     {}
 
-    fileRootNodePair(QString f, QString n)
+    fileRootNodePair(const QString &f, const QString &n)
         : file(f),
           rootNode(n)
     {}
@@ -130,10 +137,17 @@ class KERFUFFLE_EXPORT Archive : public QObject
     Q_OBJECT
 
 public:
+
+    static bool comparePlugins(const KService::Ptr &p1, const KService::Ptr &p2);
+    static QString determineMimeType(const QString& filename);
+    static KService::List findPluginOffers(const QString& filename, const QString& fixedMimeType);
+
     static Archive *create(const QString &fileName, QObject *parent = 0);
     static Archive *create(const QString &fileName, const QString &fixedMimeType, QObject *parent = 0);
     ~Archive();
 
+    ArchiveError error() const;
+    bool isValid() const;
     QString fileName() const;
     bool isReadOnly() const;
 
@@ -156,7 +170,7 @@ public:
      */
     AddJob* addFiles(const QStringList & files, const CompressionOptions& options = CompressionOptions());
 
-    ExtractJob* copyFiles(const QList<QVariant> & files, const QString & destinationDir, ExtractionOptions options = ExtractionOptions());
+    ExtractJob* copyFiles(const QList<QVariant> &files, const QString &destinationDir, const ExtractionOptions &options = ExtractionOptions());
 
     bool isSingleFolderArchive();
     QString subfolderName();
@@ -164,6 +178,7 @@ public:
 
     void setPassword(const QString &password);
     void enableHeaderEncryption(bool enable);
+    QString comment() const;
 
 private slots:
     void onListFinished(KJob*);
@@ -171,19 +186,23 @@ private slots:
     void onUserQuery(Kerfuffle::Query*);
 
 private:
-    Archive(ReadOnlyArchiveInterface *archiveInterface, QObject *parent = 0);
+    Archive(ReadOnlyArchiveInterface *archiveInterface, bool isReadOnly, QObject *parent = 0);
+    Archive(ArchiveError errorCode, QObject *parent = 0);
 
     void listIfNotListed();
     ReadOnlyArchiveInterface *m_iface;
     bool m_hasBeenListed;
+    bool m_isReadOnly;
     bool m_isPasswordProtected;
     bool m_isSingleFolderArchive;
+
     QString m_subfolderName;
     qlonglong m_extractedFilesSize;
+    ArchiveError m_error;
 };
 
-KERFUFFLE_EXPORT QStringList supportedMimeTypes();
-KERFUFFLE_EXPORT QStringList supportedWriteMimeTypes();
+KERFUFFLE_EXPORT QSet<QString> supportedMimeTypes();
+KERFUFFLE_EXPORT QSet<QString> supportedWriteMimeTypes();
 KERFUFFLE_EXPORT QSet<QString> supportedEncryptEntriesMimeTypes();
 KERFUFFLE_EXPORT QSet<QString> supportedEncryptHeaderMimeTypes();
 } // namespace Kerfuffle
