@@ -26,8 +26,8 @@
  * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
-#include "app/logging.h"
 #include "cliinterface.h"
+#include "ark_debug.h"
 #include "queries.h"
 
 #ifdef Q_OS_WIN
@@ -84,6 +84,37 @@ CliInterface::~CliInterface()
     Q_ASSERT(!m_process);
 }
 
+bool CliInterface::isCliBased() const
+{
+    return true;
+}
+
+bool CliInterface::findExecutables(bool isReadWrite)
+{
+    cacheParameterList();
+
+    QList<int> execTypes = QList<int>() << ListProgram << ExtractProgram;
+    if (isReadWrite) {
+        execTypes << AddProgram << DeleteProgram;
+    }
+
+    foreach (const int execType, execTypes) {
+        bool execTypeFound = false;
+        foreach (const QString &program, m_param.value(execType).toStringList()) {
+            if (!QStandardPaths::findExecutable(program).isEmpty()) {
+                qCDebug(ARK) << "Found executable type" << execType << ":" << program;
+                execTypeFound = true;
+                break;
+            }
+        }
+        if (!execTypeFound) {
+            return false;
+        }
+    }
+    return true;
+}
+
+
 void CliInterface::setListEmptyLines(bool emptyLines)
 {
     m_listEmptyLines = emptyLines;
@@ -91,6 +122,7 @@ void CliInterface::setListEmptyLines(bool emptyLines)
 
 bool CliInterface::list()
 {
+    resetParsing();
     cacheParameterList();
     m_operationMode = List;
 
@@ -106,9 +138,9 @@ bool CliInterface::list()
     return true;
 }
 
-bool CliInterface::copyFiles(const QVariantList &files, const QString &destinationDirectory, ExtractionOptions options)
+bool CliInterface::copyFiles(const QVariantList &files, const QString &destinationDirectory, const ExtractionOptions &options)
 {
-    qCDebug(KERFUFFLE) << Q_FUNC_INFO << "to" << destinationDirectory;
+    qCDebug(ARK) << Q_FUNC_INFO << "to" << destinationDirectory;
 
     cacheParameterList();
 
@@ -120,7 +152,7 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
     //now replace the various elements in the list
     for (int i = 0; i < args.size(); ++i) {
         QString argument = args.at(i);
-        qCDebug(KERFUFFLE) << "Processing argument " << argument;
+        qCDebug(ARK) << "Processing argument " << argument;
 
         if (argument == QLatin1String( "$Archive" )) {
             args[i] = filename();
@@ -130,7 +162,7 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
             QStringList replacementFlags = m_param.value(PreservePathSwitch).toStringList();
             Q_ASSERT(replacementFlags.size() == 2);
 
-            bool preservePaths = options.value(QLatin1String( "PreservePaths" )).toBool();
+            bool preservePaths = options.value(QStringLiteral( "PreservePaths" )).toBool();
             QString theReplacement;
             if (preservePaths) {
                 theReplacement = replacementFlags.at(0);
@@ -158,9 +190,9 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
 
             //if we get a hint about this being a password protected archive, ask about
             //the password in advance.
-            if ((options.value(QLatin1String("PasswordProtectedHint")).toBool()) &&
+            if ((options.value(QStringLiteral("PasswordProtectedHint")).toBool()) &&
                 (password().isEmpty())) {
-                qCDebug(KERFUFFLE) << "Password hint enabled, querying user";
+                qCDebug(ARK) << "Password hint enabled, querying user";
 
                 Kerfuffle::PasswordNeededQuery query(filename());
                 emit userQuery(&query);
@@ -205,9 +237,9 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
             args.removeAt(i);
 
             QString rootNode;
-            if (options.contains(QLatin1String( "RootNode" ))) {
-                rootNode = options.value(QLatin1String( "RootNode" )).toString();
-                qCDebug(KERFUFFLE) << "Set root node " << rootNode;
+            if (options.contains(QStringLiteral( "RootNode" ))) {
+                rootNode = options.value(QStringLiteral( "RootNode" )).toString();
+                qCDebug(ARK) << "Set root node " << rootNode;
             }
 
             if (!rootNode.isEmpty()) {
@@ -243,9 +275,9 @@ bool CliInterface::copyFiles(const QVariantList &files, const QString &destinati
     QTemporaryDir tmpExtractDir;
 
     if (useTmpExtractDir) {
-        qCDebug(KERFUFFLE) << "Using temporary extraction dir:" << tmpExtractDir.path();
+        qCDebug(ARK) << "Using temporary extraction dir:" << tmpExtractDir.path();
         if (!tmpExtractDir.isValid()) {
-            qCDebug(KERFUFFLE) << "Creation of temporary directory failed.";
+            qCDebug(ARK) << "Creation of temporary directory failed.";
             failOperation();
             return false;
         }
@@ -274,10 +306,10 @@ bool CliInterface::addFiles(const QStringList & files, const CompressionOptions&
 
     m_operationMode = Add;
 
-    const QString globalWorkDir = options.value(QLatin1String( "GlobalWorkDir" )).toString();
+    const QString globalWorkDir = options.value(QStringLiteral( "GlobalWorkDir" )).toString();
     const QDir workDir = globalWorkDir.isEmpty() ? QDir::current() : QDir(globalWorkDir);
     if (!globalWorkDir.isEmpty()) {
-        qCDebug(KERFUFFLE) << "GlobalWorkDir is set, changing dir to " << globalWorkDir;
+        qCDebug(ARK) << "GlobalWorkDir is set, changing dir to " << globalWorkDir;
         QDir::setCurrent(globalWorkDir);
     }
 
@@ -287,7 +319,7 @@ bool CliInterface::addFiles(const QStringList & files, const CompressionOptions&
     //now replace the various elements in the list
     for (int i = 0; i < args.size(); ++i) {
         const QString argument = args.at(i);
-        qCDebug(KERFUFFLE) << "Processing argument " << argument;
+        qCDebug(ARK) << "Processing argument " << argument;
 
         if (argument == QLatin1String( "$Archive" )) {
             args[i] = filename();
@@ -334,7 +366,7 @@ bool CliInterface::addFiles(const QStringList & files, const CompressionOptions&
             //we will decrement i afterwards
             args.removeAt(i);
 
-            QString enabled = isHeaderEncryptionEnabled() ? QLatin1String("on") : QLatin1String("off");
+            QString enabled = isHeaderEncryptionEnabled() ? QStringLiteral("-mhe=on") : QString();
 
             QStringList theSwitch = m_param.value(EncryptHeaderSwitch).toStringList();
             for (int j = 0; j < theSwitch.size(); ++j) {
@@ -388,7 +420,7 @@ bool CliInterface::deleteFiles(const QList<QVariant> & files)
     //now replace the various elements in the list
     for (int i = 0; i < args.size(); ++i) {
         QString argument = args.at(i);
-        qCDebug(KERFUFFLE) << "Processing argument " << argument;
+        qCDebug(ARK) << "Processing argument " << argument;
 
         if (argument == QLatin1String( "$Archive" )) {
             args[i] = filename();
@@ -421,14 +453,14 @@ bool CliInterface::runProcess(const QStringList& programNames, const QStringList
             break;
     }
     if (programPath.isEmpty()) {
-        const QString names = programNames.join(QLatin1String(", "));
+        const QString names = programNames.join(QStringLiteral(", "));
         emit error(xi18ncp("@info", "Failed to locate program <filename>%2</filename> on disk.",
                            "Failed to locate programs <filename>%2</filename> on disk.", programNames.count(), names));
         emit finished(false);
         return false;
     }
 
-    qCDebug(KERFUFFLE) << "Executing" << programPath << arguments;
+    qCDebug(ARK) << "Executing" << programPath << arguments;
 
     if (m_process) {
         m_process->waitForFinished();
@@ -468,7 +500,7 @@ bool CliInterface::runProcess(const QStringList& programNames, const QStringList
 
 void CliInterface::processFinished(int exitCode, QProcess::ExitStatus exitStatus)
 {
-    qCDebug(KERFUFFLE) << "Process finished, exitcode:" << exitCode << "exitstatus:" << exitStatus;
+    qCDebug(ARK) << "Process finished, exitcode:" << exitCode << "exitstatus:" << exitStatus;
 
     //if the m_process pointer is gone, then there is nothing to worry
     //about here
@@ -501,7 +533,7 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
     // Move extracted files from a QTemporaryDir to the final destination.
 
     QDir finalDestDir(finalDest);
-    qCDebug(KERFUFFLE) << "Setting final dir to" << finalDest;
+    qCDebug(ARK) << "Setting final dir to" << finalDest;
 
     bool overwriteAll = false;
     bool skipAll = false;
@@ -516,14 +548,14 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
 
             // For directories, just create the path.
             if (!finalDestDir.mkpath(relEntry.filePath())) {
-                qCWarning(KERFUFFLE) << "Failed to create directory" << relEntry.filePath() << "in final destination.";
+                qCWarning(ARK) << "Failed to create directory" << relEntry.filePath() << "in final destination.";
             }
 
         } else {
 
             // If destination file exists, prompt the user.
             if (absDestEntry.exists()) {
-                qCWarning(KERFUFFLE) << "File" << absDestEntry.absoluteFilePath() << "exists.";
+                qCWarning(ARK) << "File" << absDestEntry.absoluteFilePath() << "exists.";
 
                 if (!skipAll && !overwriteAll) {
 
@@ -537,7 +569,7 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
                             overwriteAll = true;
                         }
                         if (!QFile::remove(absDestEntry.absoluteFilePath())) {
-                            qCWarning(KERFUFFLE) << "Failed to remove" << absDestEntry.absoluteFilePath();
+                            qCWarning(ARK) << "Failed to remove" << absDestEntry.absoluteFilePath();
                         }
 
                     } else if (query.responseSkip() || query.responseAutoSkip()) {
@@ -547,7 +579,7 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
                         continue;
 
                     } else if (query.responseCancelled()) {
-                        qCDebug(KERFUFFLE) << "Copy action cancelled.";
+                        qCDebug(ARK) << "Copy action cancelled.";
                         return false;
                     }
 
@@ -557,7 +589,7 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
                 } else if (overwriteAll) {
 
                     if (!QFile::remove(absDestEntry.absoluteFilePath())) {
-                        qCWarning(KERFUFFLE) << "Failed to remove" << absDestEntry.absoluteFilePath();
+                        qCWarning(ARK) << "Failed to remove" << absDestEntry.absoluteFilePath();
                     }
                 }
 
@@ -565,12 +597,12 @@ bool CliInterface::moveToFinalDest(const QVariantList &files, const QString &fin
 
             // Create any parent directories.
             if (!finalDestDir.mkpath(relEntry.path())) {
-                qCWarning(KERFUFFLE) << "Failed to create parent dirctory for file:" << absDestEntry.filePath();
+                qCWarning(ARK) << "Failed to create parent dirctory for file:" << absDestEntry.filePath();
             }
 
             // Move files to the final destination.
             if (!QFile(absSourceEntry.absoluteFilePath()).rename(absDestEntry.absoluteFilePath())) {
-                qCWarning(KERFUFFLE) << "Failed to move file" << absSourceEntry.filePath() << "to final destination.";
+                qCWarning(ARK) << "Failed to move file" << absSourceEntry.filePath() << "to final destination.";
                 return false;
             }
         }
@@ -628,9 +660,9 @@ void CliInterface::readStdout(bool handleAll)
 
     bool foundErrorMessage =
         (wrongPasswordMessage ||
-         checkForErrorMessage(QLatin1String( lines.last() ), ExtractionFailedPatterns) ||
+         checkForErrorMessage(QLatin1String(lines.last()), ExtractionFailedPatterns) ||
          checkForPasswordPromptMessage(QLatin1String(lines.last())) ||
-         checkForFileExistsMessage(QLatin1String( lines.last() )));
+         checkForErrorMessage(QLatin1String(lines.last()), FileExistsExpression));
 
     if (foundErrorMessage) {
         handleAll = true;
@@ -673,7 +705,7 @@ void CliInterface::handleLine(const QString& line)
         //read the percentage
         int pos = line.indexOf(QLatin1Char( '%' ));
         if (pos != -1 && pos > 1) {
-            int percentage = line.mid(pos - 2, 2).toInt();
+            int percentage = line.midRef(pos - 2, 2).toInt();
             emit progress(float(percentage) / 100);
             return;
         }
@@ -681,7 +713,7 @@ void CliInterface::handleLine(const QString& line)
 
     if (m_operationMode == Copy) {
         if (checkForPasswordPromptMessage(line)) {
-            qCDebug(KERFUFFLE) << "Found a password prompt";
+            qCDebug(ARK) << "Found a password prompt";
 
             Kerfuffle::PasswordNeededQuery query(filename());
             emit userQuery(&query);
@@ -702,7 +734,7 @@ void CliInterface::handleLine(const QString& line)
         }
 
         if (checkForErrorMessage(line, WrongPasswordPatterns)) {
-            qCWarning(KERFUFFLE) << "Wrong password!";
+            qCWarning(ARK) << "Wrong password!";
             setPassword(QString());
             emit error(i18nc("@info", "Extraction failed: Incorrect password"));
             failOperation();
@@ -710,7 +742,7 @@ void CliInterface::handleLine(const QString& line)
         }
 
         if (checkForErrorMessage(line, ExtractionFailedPatterns)) {
-            qCWarning(KERFUFFLE) << "Error in extraction!!";
+            qCWarning(ARK) << "Error in extraction!!";
             emit error(i18n("Extraction failed because of an unexpected error."));
             failOperation();
             return;
@@ -723,7 +755,7 @@ void CliInterface::handleLine(const QString& line)
 
     if (m_operationMode == List) {
         if (checkForPasswordPromptMessage(line)) {
-            qCDebug(KERFUFFLE) << "Found a password prompt";
+            qCDebug(ARK) << "Found a password prompt";
 
             Kerfuffle::PasswordNeededQuery query(filename());
             emit userQuery(&query);
@@ -744,7 +776,7 @@ void CliInterface::handleLine(const QString& line)
         }
 
         if (checkForErrorMessage(line, WrongPasswordPatterns)) {
-            qCWarning(KERFUFFLE) << "Wrong password!";
+            qCWarning(ARK) << "Wrong password!";
             setPassword(QString());
             emit error(i18n("Incorrect password."));
             failOperation();
@@ -752,12 +784,25 @@ void CliInterface::handleLine(const QString& line)
         }
 
         if (checkForErrorMessage(line, ExtractionFailedPatterns)) {
-            qCWarning(KERFUFFLE) << "Error in extraction!!";
+            qCWarning(ARK) << "Error in extraction!!";
             emit error(i18n("Extraction failed because of an unexpected error."));
             failOperation();
             return;
         }
 
+        if (checkForErrorMessage(line, CorruptArchivePatterns)) {
+            qCWarning(ARK) << "Archive corrupt";
+            setCorrupt(true);
+            Kerfuffle::LoadCorruptQuery query(filename());
+            emit userQuery(&query);
+            query.waitForResponse();
+            if (!query.responseYes()) {
+                emit cancelled();
+                failOperation();
+                return;
+            }
+        }
+
         if (handleFileExistsMessage(line)) {
             return;
         }
@@ -774,23 +819,11 @@ bool CliInterface::checkForPasswordPromptMessage(const QString& line)
     if (passwordPromptPattern.isEmpty())
         return false;
 
-    if (m_passwordPromptPattern.isEmpty()) {
+    if (m_passwordPromptPattern.pattern().isEmpty()) {
         m_passwordPromptPattern.setPattern(m_param.value(PasswordPromptPattern).toString());
     }
 
-    if (m_passwordPromptPattern.indexIn(line) != -1) {
-        return true;
-    }
-
-    return false;
-}
-
-bool CliInterface::checkForFileExistsMessage(const QString& line)
-{
-    if (m_existsPattern.isEmpty()) {
-        m_existsPattern.setPattern(m_param.value(FileExistsExpression).toString());
-    }
-    if (m_existsPattern.indexIn(line) != -1) {
+    if (m_passwordPromptPattern.match(line).hasMatch()) {
         return true;
     }
 
@@ -806,21 +839,21 @@ bool CliInterface::handleFileExistsMessage(const QString& line)
 
         if (rxMatch.hasMatch()) {
             m_storedFileName = rxMatch.captured(1);
-            qCWarning(KERFUFFLE) << "Detected existing file:" << m_storedFileName;
+            qCWarning(ARK) << "Detected existing file:" << m_storedFileName;
         }
     }
 
-    if (!checkForFileExistsMessage(line)) {
+    if (!checkForErrorMessage(line, FileExistsExpression)) {
         return false;
     }
 
     Kerfuffle::OverwriteQuery query(QDir::current().path() + QLatin1Char( '/' ) + m_storedFileName);
     query.setNoRenameMode(true);
     emit userQuery(&query);
-    qCDebug(KERFUFFLE) << "Waiting response";
+    qCDebug(ARK) << "Waiting response";
     query.waitForResponse();
 
-    qCDebug(KERFUFFLE) << "Finished response";
+    qCDebug(ARK) << "Finished response";
 
     QString responseToProcess;
     const QStringList choices = m_param.value(FileExistsInput).toStringList();
@@ -851,7 +884,7 @@ bool CliInterface::handleFileExistsMessage(const QString& line)
 
 bool CliInterface::checkForErrorMessage(const QString& line, int parameterIndex)
 {
-    QList<QRegExp> patterns;
+    QList<QRegularExpression> patterns;
 
     if (m_patternCache.contains(parameterIndex)) {
         patterns = m_patternCache.value(parameterIndex);
@@ -861,13 +894,13 @@ bool CliInterface::checkForErrorMessage(const QString& line, int parameterIndex)
         }
 
         foreach(const QString& rawPattern, m_param.value(parameterIndex).toStringList()) {
-            patterns << QRegExp(rawPattern);
+            patterns << QRegularExpression(rawPattern);
         }
         m_patternCache[parameterIndex] = patterns;
     }
 
-    foreach(const QRegExp& pattern, patterns) {
-        if (pattern.indexIn(line) != -1) {
+    foreach(const QRegularExpression& pattern, patterns) {
+        if (pattern.match(line).hasMatch()) {
             return true;
         }
     }
@@ -947,7 +980,7 @@ void CliInterface::writeToProcess(const QByteArray& data)
     Q_ASSERT(m_process);
     Q_ASSERT(!data.isNull());
 
-    qCDebug(KERFUFFLE) << "Writing" << data << "to the process";
+    qCDebug(ARK) << "Writing" << data << "to the process";
 
 #ifdef Q_OS_WIN
     m_process->write(data);
