@@ -355,6 +355,8 @@ void Part::updateActions()
                                 (selectedEntriesCount == 1));
     m_extractFilesAction->setEnabled(!isBusy() &&
                                      (m_model->rowCount() > 0));
+    m_saveAsAction->setEnabled(!isBusy() &&
+                               m_model->rowCount() > 0);
     m_addFilesAction->setEnabled(!isBusy() &&
                                  isWritable);
     m_addDirAction->setEnabled(!isBusy() &&
@@ -365,11 +367,11 @@ void Part::updateActions()
     m_openFileAction->setEnabled(!isBusy() &&
                                  isPreviewable &&
                                  !isDirectory &&
-                                 (selectedEntriesCount > 0));
+                                 (selectedEntriesCount == 1));
     m_openFileWithAction->setEnabled(!isBusy() &&
                                      isPreviewable &&
                                      !isDirectory &&
-                                     (selectedEntriesCount > 0));
+                                     (selectedEntriesCount == 1));
 
     QMenu *menu = m_extractFilesAction->menu();
     if (!menu) {
